diff -ruN uppsrc/ide/LayDes/laydes.cpp uppsrc/ide/LayDes/laydes.cpp
--- uppsrc/ide/LayDes/laydes.cpp	2013-09-02 20:47:56 +0400
+++ uppsrc/ide/LayDes/laydes.cpp	2013-09-04 04:38:44 +0400
@@ -1297,6 +1297,46 @@
 	ReloadItems();
 }
 
+bool RectTopLeftLess(const Rect& a, const Rect& b)
+{
+	if(a.top < b.top)
+		return true;
+	if(a.top == b.top)
+		return a.left < b.left;
+	return false;
+}
+
+void LayDes::SortTopLeftItems()
+{
+	SaveState();
+	int count = cursor.GetCount();
+	if(currentlayout < 0 || count < 2)
+		return;
+	LayoutData& l = CurrentLayout();
+	Array<LayoutItem> item;
+	item.SetCount(count);
+	Vector<Rect> rect;
+	rect.SetCount(count);
+	Vector<int> order;
+	order <<= cursor;
+	for(int i = 0; i < count; ++i) {
+		rect[i] = CtrlRect(l.item[cursor[i]].pos, l.size);
+		item[i] = l.item[cursor[i]];
+	}
+	IndexSort2(rect, order, item, &RectTopLeftLess);
+	int q = FindIndex(order, cursor.Top());
+	Sort(cursor);
+	l.item.Remove(cursor);
+	int ii = cursor[0];
+	l.item.InsertPick(ii, item);
+	cursor.Clear();
+	for(int i = 0; i < count; ++i)
+		cursor.Add(i + ii);
+	if(q >= 0 && q != count - 1)
+		Swap(cursor.Top(), cursor[q]);
+	ReloadItems();
+}
+
 void LayDes::Flush()
 {
 	currentlayout = -1;
@@ -1489,9 +1529,18 @@
 		}
 	}
 	else if(item.IsCursor()) {
-		if(!GetCtrl())
+		int c = item.GetCursor();
+		if(!GetCtrl()) {
 			cursor.Clear();
-		cursor.Add(item.GetCursor());
+			cursor.Add(c);
+		}
+		else {
+			int q = FindIndex(cursor, c);
+			if(q >= 0)
+				cursor.Remove(q);
+			else
+				cursor.Add(c);
+		}
 	}
 	SetFocus();
 	SyncItems();
diff -ruN uppsrc/ide/LayDes/LayDes.h uppsrc/ide/LayDes/LayDes.h
--- uppsrc/ide/LayDes/LayDes.h	2013-08-21 15:14:38 +0400
+++ uppsrc/ide/LayDes/LayDes.h	2013-09-04 04:36:31 +0400
@@ -385,6 +385,7 @@
 	void        MoveUp();
 	void        MoveDown();
 	void        SortItems();
+	void        SortTopLeftItems();
 	void        SelectAll();
 
 	void        Matrix();
diff -ruN uppsrc/ide/LayDes/laydes.key uppsrc/ide/LayDes/laydes.key
--- uppsrc/ide/LayDes/laydes.key	2013-08-21 15:14:38 +0400
+++ uppsrc/ide/LayDes/laydes.key	2013-09-04 04:33:39 +0400
@@ -3,6 +3,7 @@
 KEY(MOVEUP, "Move Up", K_CTRL_UP)
 KEY(MOVEDOWN, "Move Down", K_CTRL_DOWN)
 KEY(SORTITEMS, "Sort by position", 0)
+KEY(SORTTOPLEFTITEMS, "Sort by top-left position", 0)
 KEY(LAYOUTS, "Layouts..", 0)
 KEY(PREVLAYOUT, "Previous layout", K_ALT_PAGEUP)
 KEY(NEXTLAYOUT, "Next layout", K_ALT_PAGEDOWN)
diff -ruN uppsrc/ide/LayDes/laywin.cpp uppsrc/ide/LayDes/laywin.cpp
--- uppsrc/ide/LayDes/laywin.cpp	2013-08-21 15:14:38 +0400
+++ uppsrc/ide/LayDes/laywin.cpp	2013-09-04 04:43:05 +0400
@@ -48,7 +48,10 @@
 	bool iscursor = currentlayout >= 0 && cursor.GetCount();
 	bar.Add(iscursor, AK_MOVEUP, LayImg::MoveUp(), THISBACK(MoveUp));
 	bar.Add(iscursor, AK_MOVEDOWN, LayImg::MoveDown(), THISBACK(MoveDown));
-	bar.Add(cursor.GetCount() >= 2, AK_SORTITEMS, LayImg::SortItems(), THISBACK(SortItems));
+	bar.Separator();
+	bool sortable = cursor.GetCount() >= 2;
+	bar.Add(sortable, AK_SORTITEMS, LayImg::SortItems(), THISBACK(SortItems));
+	bar.Add(sortable, AK_SORTTOPLEFTITEMS, LayImg::SortItems(), THISBACK(SortTopLeftItems));
 }
 
 void LayDes::AlignBar(Bar& bar)
